<!--- 
	csw-tabs-horizontal.cfc
	
	Version: [2018-01-03 10:08 am]
---> 

<cfcomponent displayname="csw-tabs-horizontal" output="false" hint="Horizontal Tabs Functions" extends="_cs_widgets._common.csw-common">

	<cffunction name="loadResources" access="public" output="Yes" returntype="void">
		<cfscript>
			Server.CommonSpot.udf.resources.loadResources("jQuery");
			Server.CommonSpot.udf.resources.loadResources("FontAwesome");
			Server.CommonSpot.udf.resources.loadResources("cswTabsHorizontal");
			Server.CommonSpot.udf.resources.loadResources("cswCommonStyles");
			Server.CommonSpot.udf.resources.loadResources("cswTextStyles");

			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswCommonStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswCommonStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswTextStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswTextStyles_custom");
			if ( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswTabsHorizontal_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswTabsHorizontal_custom");
		</cfscript>
	</cffunction>
	

	<cffunction name="loadProperties" access="public" returntype="struct" output="No">
		<cfargument name="elementInfo" type="struct" required="Yes">
		<cfargument name="rhCaller" type="struct" required="false" default="#StructNew()#">
		<cfargument name="showDataError" type="boolean" required="false" default="true">
		
		<cfscript>
			var widgetMetadataFormName = "CSW Tabs Horizontal";
			var props = StructNew();
			var rhMetadata = arguments.ElementInfo.RenderHandlerMetaData;
			var widgetData = StructNew();
			
			
			// Check for Props and Layout Binding
			if( NOT hasPropsLayoutBinding(ElementInfo=arguments.elementInfo,metadataFormName=widgetMetadataFormName,rhCaller=arguments.rhCaller) )
				return props;

			//
			// Get Metadata (Horizontal Tabs metadata)
			// ----------------------------------------------
			widgetData = rhMetadata[widgetMetadataFormName];


			/********************************************/
			/**** TAB & CONTENT PROPERTIES TAB **********/
			/********************************************/

			// Tab Type 
			props.tabType = getStandardField( widgetData=widgetData, field='Tab_Type', defValue='nav-tabs' );

			// Tab Bar Style
			props.tabBar = getFieldSettings( widgetData=widgetData, field='Tab_Bar' );			

			// Tab Bar Line 
			props.tabBarLineFullWidth = getStandardField( widgetData=widgetData, field='Line_Below_Full_Width', defValue='1' );
				
			// Content Area
			props.contentArea = getFieldSettings( widgetData=widgetData, field='Content_Area' );			
			

			/************************************/
			/**** DEFAULT TAB PROPERTIES TAB ****/
			/************************************/
			
			// Default Style
			props.defaultTab = getFieldSettings(widgetData=widgetData, field='Tab_Style' );
			
			// Default Padding
			props.defaultPadding = getStandardField(widgetData=widgetData, field='Tab_Padding' );

			// Border Style
			props.defaultBorder = getFieldSettings(widgetData=widgetData, field='Border_Style' );
			
			// Corners 
			props.defaultCorners = getStandardField(widgetData=widgetData, field='Corners' );
			
			// show Icon
			props.showIcon = getStandardField(widgetData=widgetData, field='Show_Icon' );
			
			// Responsive
			props.responsive = getDeviceSizeOptions(widgetData=widgetData, field='Responsive', defXSS='cswTabs-stacked-xxs', defXS='cswTabs-stacked-xs', defSM='cswTabs-stacked-sm', defMD='cswTabs-normal-md', defLG='cswTabs-normal-lg');		
			props.responsive = getDeviceSizeClassList(props.responsive);			
			
			// Circle Size
			props.circleSize = '';
			if( props.tabType contains 'cswTab-circle' )
			{
				props.circleSize = getDeviceSizeOptions(widgetData=widgetData, field='Circle_Size', defXSS='cswTabCircle-50', defXS='cswTabCircle-75', defSM='cswTabCircle-100', defMD='cswTabCircle-125', defLG='cswTabCircle-125');		
				props.circleSize = getDeviceSizeClassList(props.circleSize);
			}
			

			
			/************************************/
			/**** ACTIVE TAB PROPERTIES TAB ****/
			/************************************/

			// Active Style
			props.activeTab = getFieldSettings(widgetData=widgetData, field='Active_Tab_Style' );
			
			// Active Border Style
			props.activeBorder = getFieldSettings(widgetData=widgetData, field='Active_Border_Style' );

			// Hightlight Line (borderColor & size)
			props.highlightLine = getFieldSettings(widgetData=widgetData, field='Highlight_line' );
			
			
			/************************************/
			/**** HOVER TAB PROPERTIES TAB ****/
			/************************************/

			// Hover Style
			props.hoverTab = getFieldSettings(widgetData=widgetData, field='Hover_Tab_Style' );
			
			// Hover Border Style
			props.hoverBorder = getFieldSettings(widgetData=widgetData, field='Hover_Border_Style' );
						
			
			/**************************************/
			/**** GENERAL PROPERTIES TAB **********/
			/**************************************/

			// Element (STRUCT)
			props.element = getFieldSettings(widgetData=widgetData, field='Overall' );

			// Background Image (STRUCT)
			props.backgroundImage = getStandardMetadataImageField( widgetData=widgetData, field='Background_Image', defValue='' );

			// Background Image Focal Point
			props.backgroundImageFocalPoint = getStandardField(widgetData=widgetData, field='Background_Image_Focal_Point', defValue='cswImgPos-centermiddle');

			/***** overrides ******/
			if( props.tabType contains 'lineabove' )
				props.activeBorder = props.defaultBorder;

			if( props.tabType contains 'cswTab-circle' )		
				props.defaultPadding = '';		// clear out padding for circle

			if( FindNoCase( 'cswTabs-standard', props.tabType ) )
			{
				props.activeBorder.borderColor = '';
				props.hoverBorder.borderColor = '';
			}
			else if( FindNoCase( 'cswTabs-bottomline', props.tabType ) )
			{
				props.defaultTab.bgColor = '';
				props.activeTab.bgColor = '';
				props.hoverTab.bgColor = '';
				props.defaultCorners = 'square';
			}
			else if( props.tabType eq 'cswTabs-spacedbottomline' )
			{
				props.defaultTab.bgColor = '';
				props.hoverTab.bgColor = '';
				props.defaultCorners = 'square';
			}
			else if( FindNoCase( 'cswTabs-folders', props.tabType ) 
						OR FindNoCase( 'cswTabs-slanted', props.tabType ) )
			{
				props.defaultCorners = 'square';
				if( len(props.defaultTab.bgColor) eq 0 OR props.defaultTab.bgColor EQ 'inherit' )
					props.defaultTab.bgColor = 'grey';
			}
			
			if( props.tabType EQ 'cswTabs-standard' )
				props.tabBarLineFullWidth = 1;	// force on
			else if( FindNoCase( 'cswTabs-noborder', props.tabType ) 
					OR FindNoCase( 'cswTabs-middleborder', props.tabType )
					OR props.tabType EQ 'cswTabs-pills' )
				props.tabBarLineFullWidth = 0; // force off
				
			return props;
		</cfscript>
	</cffunction>


	
	<cffunction name="getAndRenderItems" access="public" output="Yes" returntype="void">
		<cfargument name="elementDataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">

		<cfscript>
			var elementData = arguments.elementDataStruct;
			var props = arguments.propsStruct;
			var elementID = request.element.id;
			var tabContent = ArrayNew(1);
			var styleAttr = '';
			var outerClasses = props.tabType;	
			var contentStyleAttr = '';

			// Overall element styles & classes
			if( len(props.element.padding) )
					styleAttr = styleAttr & 'padding: #props.element.padding#;';
			if( len(props.element.bgColor) )
				styleAttr = styleAttr & 'color:##fff;background-color: #props.element.bgColor#;';

			if( StructKeyExists(props,'backgroundImage') AND len(props.backgroundImage) )
				styleAttr = styleAttr & "background-image: url('#props.backgroundImage#'); background-repeat: no-repeat; background-size: cover;";

			if( len(styleAttr) )
				styleAttr = 'style="#styleAttr#"';
				
			if( len(props.backgroundImageFocalPoint) )
				outerClasses = outerClasses & ' ' & props.backgroundImageFocalPoint;	
			if( len(props.defaultCorners) )
				outerClasses = outerClasses & ' ' & props.defaultCorners;	
			if( len(props.responsive) )	
				outerClasses = outerClasses & ' ' & props.responsive;	
			if( len(props.circleSize) )
				outerClasses = outerClasses & ' ' & props.circleSize;

			// Content Area styles & classes
			if( len(props.contentArea.padding) )
				contentStyleAttr = contentStyleAttr & "padding: #props.contentArea.padding#;";
			if( len(props.contentArea.bgColor) )
				contentStyleAttr = contentStyleAttr & "background-color: #props.contentArea.bgColor#;";
			if( len(contentStyleAttr) )	
				contentStyleAttr = 'style="#contentStyleAttr#"';
		</cfscript>

		<cfoutput>
		<svg class="hidden" style="display:none;">
			<defs>
				<path id="tabshape" d="M80,60C34,53.5,64.417,0,0,0v60H80z"/>
			</defs>
		</svg>
		
		<section id="cswTabs-#elementID#" class="cswTabsHoriz #outerClasses#" #styleAttr#>
			<div class="tabs cswTabWrapper">
				<!--- render the tabs and return tab content --->
				<cfset tabContent = renderTabs(elementData,props)>
				<div id="cswWrapper-#elementID#" class="cswTabContainers" #contentStyleAttr#>
					#renderTabContainers(tabContent,props)#
				</div> 
			</div>
		</section>
		<style>
			<cfif props.tabType eq 'cswTabs-folders'>
				#renderFolderStyleBlock(props)#
			<cfelseif props.tabType eq 'cswTabs-slanted'>	
				#renderSlantedStyleBlock(props)#
			<cfelse>
				#renderStyleBlock(props)#
			</cfif>	
			
			#renderPaddingStyles(props)#
		</style>
		</cfoutput>
		
	</cffunction>



	<cffunction name="renderTabs" access="public" output="Yes" returntype="array">
		<cfargument name="dataStruct" required="Yes" type="struct">
		<cfargument name="propsStruct" required="Yes" type="struct">

		<cfscript>
			var dStruct = arguments.dataStruct.propertyvalues;
			var props = arguments.propsStruct;			
			var numData = arraylen(dStruct);
			var elementID = request.element.id;
			var tabCounter = 0;
			var idx = 0;
			var tabTitleID = 0;
			var tabID = 0;
			var tabHashID = 0;
			var tabKey = 0;
			var tabContentKey = 0;
			var tabActiveClass = '';
			var tabArray = arrayNew(1);
			var tabBarStyleAttr = '';
			var tabBarClasses = '';	
			var data = StructNew();
			var textClasses = '';	
			
			
			// Tab Bar  styles & classes
			if( len(props.tabBar.padding) )
				tabBarStyleAttr = tabBarStyleAttr & "padding: #props.tabBar.padding#;";
			if( len(props.tabBar.bgColor) )
				tabBarStyleAttr = tabBarStyleAttr & "background-color: #props.tabBar.bgColor#;";
				
			if( len(tabBarStyleAttr) )	
				tabBarStyleAttr = 'style="#tabBarStyleAttr#"';
			
			if( len(props.tabBar.align) )
				tabBarClasses = trim( tabBarClasses & ' #props.tabBar.align#');	
		</cfscript>


		<!--- // Tab Bar --->
		<cfoutput>
		<nav class="cswTabBar #tabBarClasses#" #tabBarStyleAttr#>
			<ul>
			</cfoutput>
	
			<cfloop index="idx" from="1" to="#numData#">
	
				<cfif structKeyExists(dStruct[idx].values,"title") and len(dStruct[idx].values.title)>
	
					<cfscript>
						data = StructNew();
						
						// Get TabID (Optional)
						// if it does NOT exists, then tabs are bound to content by their ORDER (Ugh!)
						if( StructKeyExists(dStruct[idx].values, 'tabID') )
							data.tabID = dStruct[idx].values.tabID;
						else
							data.tabID = idx;
							
						data.title = dStruct[idx].values.title;
						if( structKeyExists( dStruct[idx].values, 'icon') )
							data.icon = dStruct[idx].values.icon;
						else
							data.icon = '';
						
						tabCounter = tabCounter + 1;
	
						// Build the simple tabTitleID
						tabTitleID = LCase( reReplace( application.ADF.data.unescapeHTMLentities(data.title), '[^a-zA-Z0-9\-\_]', '-', 'ALL' ) );
						tabTitleID = REReplaceNoCase(tabTitleID,'-{2,}','-','all');
						tabTitleID = ListChangeDelims(tabTitleID, "-", "-");
	
						// Build the unique TabID & Set the tab DIV IDs
						tabID = tabTitleID & "-" & elementID & "-" & data.tabID;
						tabHashID = 'hTabID-' & tabTitleID;
						tabKey = "hTab-" & tabID;
						tabContentKey = "hTabContent-" & tabID;
	
						tabActiveClass = '';
						if( tabCounter EQ 1 )
							tabActiveClass = "tab-current";
							
						// text size
						if( len(props.defaultTab.size) )
							textClasses = props.defaultTab.size;
					</cfscript>
						
					<!--- Render the Tab --->	
					<cfoutput>
						<li role="presentation" class="#tabActiveClass# cswTabNavItem #textClasses# #props.showIcon#">
							<a href="###tabKey#" id="#tabHashID#">					
					</cfoutput>
	
						<cfif props.tabType neq 'cswTabs-folders'>
							<cfoutput>
								<div class="cswInner">
									<cfif len(data.icon) AND (props.showIcon eq 'cswTabIconAlign-left' OR props.showIcon eq 'cswTabIconAlign-top')>
										<i class="#data.icon# cswTabIcon"></i>
									</cfif>
									<span class="cswTabTitle">#data.title#</span>
									<cfif len(data.icon) AND props.showIcon eq 'cswTabIconAlign-right'>
										<i class="#data.icon# cswTabIcon"></i>
									</cfif>
								</div>
							</cfoutput>
							
						<cfelse>		<!---// FOLDERS --->
					
							<cfoutput>
								<cfif idx eq 1 OR idx eq numData>
									<svg viewBox="0 0 80 60" preserveAspectRatio="none"><use xlink:href="##tabshape"></use></svg>
								<cfelse>
									<svg viewBox="0 0 80 60" preserveAspectRatio="none"><use xlink:href="##tabshape"></use></svg>
									<svg viewBox="0 0 80 60" preserveAspectRatio="none"><use xlink:href="##tabshape"></use></svg>
								</cfif>
								
								<div class="cswInner">	
									<cfif len(data.icon) AND (props.showIcon eq 'cswTabIconAlign-left' OR props.showIcon eq 'cswTabIconAlign-top')>
										<i class="#data.icon# cswTabIcon"></i>
									</cfif>
									<span class="cswTabTitle">#data.title#</span>
									<cfif len(data.icon) AND props.showIcon eq 'cswTabIconAlign-right'>
										<i class="#data.icon# cswTabIcon"></i>
									</cfif>
								</div>	
							</cfoutput>	
						</cfif>

					<cfoutput>
						</a>
					</li>
					</cfoutput>
						
					<cfscript>
						tabStruct = structNew();
						tabStruct.tabKey = tabKey;		// DOM ID
						tabStruct.tabContentKey = tabContentKey;
						tabStruct.title = data.title;
						tabStruct.tabID = data.tabID;	// unique ID, or the tab position if tabID not passed
	
						arrayAppend(tabArray,tabStruct);
					</cfscript>
	
				</cfif>
	
			</cfloop>
		
			<cfoutput>	
			</ul> 
		</nav>
		</cfoutput>
		
		<cfreturn tabArray>

	</cffunction>



	<cffunction name="renderTabContainers" access="public" output="Yes" returntype="string">
		<cfargument name="tabArray" required="Yes" type="array">

		<cfscript>
			var data = arguments.tabArray;
			var tabContainerHTML = "";
			var contentActiveClass = "";
			var ariaHiddenState = "true";
			var idx = 0;
		</cfscript>

		<cfsavecontent variable="tabContainerHTML">

			<!--- // Tab Content --->
			<cfoutput>
			<div class="cswContentWrap">
			</cfoutput>

				<cfloop index="idx" from="1" to="#arraylen(data)#">

					<cfscript>
						contentActiveClass = "";
						ariaHiddenState = "true";
						if ( idx EQ 1 )
						{
							ariaHiddenState = "false";
							contentActiveClass = "content-current";
						}
					</cfscript>

					<cfoutput>
					<section id="#data[idx].tabKey#" aria-hidden="#ariaHiddenState#" role="tabpanel" aria-labelledby="#data[idx].tabKey#"
						class="#contentActiveClass# cswTabContent" data-title="#XmlFormat(data[idx].title)#">
					</cfoutput>	

						<!--- // Render a container for each tab --->
						<CFMODULE TEMPLATE="/commonspot/utilities/ct-render-named-element.cfm"
									elementtype="container"
									elementName="tabContainer_#data[idx].tabID#_#request.element.id#">
									
					<cfoutput>	
					</section>
					</cfoutput>
					
				</cfloop>

			<cfoutput>	
			</div> <!--- // END - Tab Content Column --->
			</cfoutput>

		</cfsavecontent>

		<cfreturn tabContainerHTML>

	</cffunction>
	
	
	<cffunction name="GetFontStyles" access="private" output="No" returntype="string">
		<cfargument name="fontSettings" type="string" required="yes">
		
		<cfscript>
			var st = StructNew();
			var key = '';
			var value = '';
			var item = '';
			var retString = '';

			for( i=1; i lte ListLen(arguments.FontSettings); i++ )
			{
				item = ListGetAt(arguments.FontSettings, i);
				key = ListFirst(item,':');
				value = ListLast(item,':');
				st[key] = value;
			}	
			
			// Bold
			if( structKeyExists(st, 'bold') )
			{
				if( st.bold eq 1 )
					retstring = 'font-weight:bold;';
		}

			// Italic
			if( structKeyExists(st, 'italic') )
			{
				if( st.italic eq 1 )
					retstring = retstring & ' font-style:italic;';
			}
			
			// Underline			
			if( structKeyExists(st, 'underline') )
			{
				if( st.underline eq 1 )
					retstring = retstring & ' text-decoration:underline;';
			}			
			
			// Case
			if( structKeyExists(st, 'case') )
			{
				retstring = retstring & ' text-transform:#st.case#;';
			}											
			
			return retString;
		</cfscript>
		
	</cffunction>
	
	
	<cffunction name="renderStyleBlock" access="private" output="Yes" returntype="void">
		<cfargument name="propsStruct" type="struct" required="Yes">
	
		<cfscript>
			var props = arguments.propsStruct;
			var defaultFontStyles = GetFontStyles(props.defaultTab.fontSettings);
			var hoverFontStyles = GetFontStyles(props.hoverTab.fontSettings);
			var activeFontStyles = GetFontStyles(props.activeTab.fontSettings);
			var hoverBorderStruct = GetBorderStyles(props.defaultBorder, props.hoverBorder);
			var elementID = request.element.id;
		</cfscript>

		<!--- Default color & bgColor --->		
		<cfoutput>
	##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a {
			<cfif len(props.defaultTab.color)>
				color: #props.defaultTab.color#;
			</cfif>
			<cfif len(props.defaultTab.bgColor)>
				background-color: #props.defaultTab.bgColor#;
			</cfif>
			#defaultFontStyles#
		}
		</cfoutput>

		<!--- Hover  color & bgColor --->				
		<cfif props.hoverTab.show OR props.hoverBorder.show>
			<cfoutput>
			##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:hover,
			##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:focus {
				<cfif props.hoverTab.show>
					<cfif len(props.hoverTab.color)>
						color: #props.hoverTab.color#;
					</cfif>
					<cfif len(props.hoverTab.bgColor)>
						background-color: #props.hoverTab.bgColor#;
					</cfif>
					#hoverFontStyles#
				</cfif>
				<cfif props.hoverBorder.show AND len(props.hoverBorder.borderColor)>				
					border: #hoverBorderStruct.size# solid #hoverBorderStruct.borderColor#;
				</cfif>
			}
			</cfoutput>			
		</cfif>

		<!--- Active color & bgColor --->				
		<cfoutput>		
		##cswTabs-#elementID# > div > nav > ul > li.tab-current > a {
			<cfif len(props.activeTab.color)>
				color: #props.activeTab.color#;
			</cfif>
			<cfif len(props.activeTab.bgColor)>
				background-color: #props.activeTab.bgColor#;
			</cfif>
			#activeFontStyles#				
		}
		</cfoutput>	

		<!--- Border Styles --->		
		<cfoutput>#RenderBorderStyles(props)#</cfoutput>
		
	</cffunction>
	
	

	<cffunction name="renderSlantedStyleBlock" access="private" output="Yes" returntype="void">
		<cfargument name="propsStruct" type="struct" required="Yes">
	
		<cfscript>
			var props = arguments.propsStruct;
			var defaultFontStyles = GetFontStyles(props.defaultTab.fontSettings);
			var hoverFontStyles = GetFontStyles(props.hoverTab.fontSettings);
			var activeFontStyles = GetFontStyles(props.activeTab.fontSettings);
			var elementID = request.element.id;
		</cfscript>

		<!--- Default color & bgColor --->		
		<cfoutput>
		##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a {
			<cfif len(props.defaultTab.color)>
				color: #props.defaultTab.color#;
			</cfif>
			#defaultFontStyles#
		}
		##cswTabs-#elementID# > div > nav > ul > li.cswTabIconAlign-right > a > i,
		##cswTabs-#elementID# > div > nav > ul > li.cswTabIconAlign-left > a > span,
		##cswTabs-#elementID# > div > nav > ul > li.cswTabIconAlign-top > a > span,
		##cswTabs-#elementID# > div > nav > ul > li.cswTabIconAlign-none > a > span {	
			margin-right:20px;
		}
		</cfoutput>

		<!--- Hover  color & bgColor --->				
		<cfif props.hoverTab.show OR props.hoverBorder.show>
			<cfoutput>
			##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:hover,
			##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:focus {
				<cfif props.hoverTab.show>
					<cfif len(props.hoverTab.color)>
						color: #props.hoverTab.color#;
					</cfif>
					#hoverFontStyles#
				</cfif>
			}
			</cfoutput>			
		</cfif>

		<!--- Active color & bgColor --->				
		<cfoutput>		
		##cswTabs-#elementID# > div > nav > ul > li.tab-current > a {
			<cfif len(props.activeTab.color)>
				color: #props.activeTab.color#;
			</cfif>
			#activeFontStyles#				
		}
		</cfoutput>	

		<!--- Border Styles --->		
		<cfoutput>#RenderBorderStyles(props)#</cfoutput>
		
	</cffunction>
	

	<cffunction name="renderFolderStyleBlock" access="private" output="Yes" returntype="void">
		<cfargument name="propsStruct" type="struct" required="Yes">
	
		<cfscript>
			var props = arguments.propsStruct;
			var defaultFontStyles = GetFontStyles(props.defaultTab.fontSettings);
			var hoverFontStyles = GetFontStyles(props.hoverTab.fontSettings);
			var activeFontStyles = GetFontStyles(props.activeTab.fontSettings);
			var hoverBorderStruct = GetBorderStyles(props.defaultBorder, props.hoverBorder);
			var elementID = request.element.id;
		</cfscript>

		<!--- Default color & bgColor --->		
		<cfoutput>
		##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a {
			<cfif len(props.defaultTab.color)>
				color: #props.defaultTab.color#;
			</cfif>
			<cfif len(props.defaultTab.bgColor)>
				background-color: #props.defaultTab.bgColor#;
			</cfif>
			#defaultFontStyles#
		}
		</cfoutput>

		<!--- Hover  color & bgColor --->				
		<cfif props.hoverTab.show OR props.hoverBorder.show>
			<cfoutput>
			##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:hover,
			##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:focus {
				<cfif props.hoverTab.show>
					<cfif len(props.hoverTab.color)>
						color: #props.hoverTab.color#;
					</cfif>
					<cfif len(props.hoverTab.bgColor)>
						background-color: #props.hoverTab.bgColor#;
					</cfif>
					#hoverFontStyles#
				</cfif>
				<cfif props.hoverBorder.show AND len(props.hoverBorder.borderColor)>				
					border: #hoverBorderStruct.size# solid #hoverBorderStruct.borderColor#;
				</cfif>
			}
			</cfoutput>			
		</cfif>

		<!--- Active color & bgColor --->				
		<cfoutput>		
		##cswTabs-#elementID# > div > nav > ul > li.tab-current > a {
			<cfif len(props.activeTab.color)>
				color: #props.activeTab.color#;
			</cfif>
			<cfif len(props.activeTab.bgColor)>
				background-color: #props.activeTab.bgColor#;
			</cfif>
			#activeFontStyles#				
		}
		</cfoutput>	

		<!--- Border Styles --->		
		<cfoutput>#RenderBorderStyles(props)#</cfoutput>
		
	</cffunction>


	<cffunction name="renderBorderStyles" access="private" output="yes" returntype="void">
		<cfargument name="propsStruct" type="struct" required="Yes">
		
		<cfscript>
			var props = arguments.propsStruct;
			var activeStruct = getBorderStyles(props.defaultBorder, props.activeBorder);
			var hoverStruct = getBorderStyles(props.defaultBorder, props.hoverBorder);
			var elementID = request.element.id;
			var lineColor = '';
		</cfscript>
		
		<!--- STANDARD --->			
		<cfif props.tabType contains 'cswTabs-standard'>
			<cfoutput>
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a {
					border-bottom: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#;		
					margin-right: 3px;
					height: 100%;
					padding: #props.defaultBorder.size#;
					margin-bottom: -#props.defaultBorder.size#;
				}
				
				<!--- HOVER --->
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:hover,
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:focus {
					border: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#;
					padding: 0;
				}
				
				<!--- Current Tab --->
				##cswTabs-#elementID# > div > nav > ul > li.tab-current > a,
				##cswTabs-#elementID# > div > nav > ul > li.tab-current > a:hover,
				##cswTabs-#elementID# > div > nav > ul > li.tab-current > a:focus {
					margin-right: 3px;
					border: #props.defaultBorder.size# solid transparent;		
					<cfif len(props.activeTab.bgColor) AND props.activeTab.bgColor neq 'inherit' >
						border-color: #props.defaultBorder.borderColor# #props.defaultBorder.borderColor# #props.activeTab.bgColor# #props.defaultBorder.borderColor#;
					<cfelse>
						border-color: #props.defaultBorder.borderColor# #props.defaultBorder.borderColor# transparent #props.defaultBorder.borderColor#;
					</cfif>
				}
			</cfoutput>

		<!--- FULL --->				
		<cfelseif props.tabType contains 'cswTabs-fullborder'>
			<cfoutput>
				##cswTabs-#elementID# > div > nav > ul > li > a {
					border: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#;
				}					
				/* default tab following active tab make left border active color */
				##cswTabs-#elementID# > div > nav > ul > li.tab-current + li > a {
					border-left: #props.defaultBorder.size# solid #props.activeBorder.borderColor#;
				}
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current):hover + li > a,
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current):focus + li > a {
					border-left: #props.defaultBorder.size# solid #props.hoverBorder.borderColor#;
				}
				
				/* hover */
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:hover,
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:focus {
					border: #props.defaultBorder.size# solid #hoverStruct.borderColor#;
				}
				
				<cfif props.tabType eq 'cswTabs-fullborder'>
				##cswTabs-#elementID# > div > nav > ul > li.tab-current > a {
					border: #props.defaultBorder.size# solid #props.activeBorder.borderColor#;
				}					
				</cfif>
				
				##cswTabs-#elementID# > div > nav > ul > li:first-child > a {
					margin-left: 0;
				}
				##cswTabs-#elementID# > div > nav > ul > li:not(:first-child) > a {
					margin-left: -#props.defaultBorder.size#;
				}
			</cfoutput>
			
		<!--- NONE --->				
		<cfelseif props.tabType contains 'cswTabs-noborder'>
			<cfoutput>
				##cswTabs-#elementID# > div > nav > ul > li > a,
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:hover,
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:focus,
				##cswTabs-#elementID# > div > nav > ul > li.tab-current > a {
					border: none;
				}
			</cfoutput>
		
		<!--- SLANTED--->          
		<cfelseif props.tabType eq 'cswTabs-slanted'>
			<cfoutput>
				##cswTabs-#elementID# > div > nav > ul > li > a::after {
			    	background: #props.defaultTab.bgColor#;	
				}	

				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:hover::after,
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:focus::after {
			    	background: #props.hoverTab.bgColor#;	
				}	
		
				##cswTabs-#elementID# > div > nav > ul > li.tab-current > a::after {
			    	background: #props.activeTab.bgColor#;		
				}
			</cfoutput>
					
		<!--- PILLS --->				
		<cfelseif props.tabType contains 'cswTabs-pills'>
			<cfoutput>
				##cswTabs-#elementID# > div > nav > ul > li > a,
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:hover,
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:focus,
				##cswTabs-#elementID# > div > nav > ul > li.tab-current > a {
					border: none;
					border-radius: 4px;
					margin-right: 3px;
				}
			</cfoutput>

		<!--- MIDDLE BORDER --->				
		<cfelseif props.tabType contains 'cswTabs-middleborder'>
			<cfoutput>
				##cswTabs-#elementID# > div > nav > ul > li > a {
					border-left: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#;
					border-right: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#;
				}		
							
				/* default tab following active tab make left border active color */
				##cswTabs-#elementID# > div > nav > ul > li.tab-current + li > a {
					border-left: #props.defaultBorder.size# solid #props.activeBorder.borderColor#;
				}
				
				/* hover */
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:hover,
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:focus {
					border-left: #props.defaultBorder.size# solid #hoverStruct.borderColor#;
					border-right: #props.defaultBorder.size# solid #hoverStruct.borderColor#;
				}
				
				<cfif props.tabType eq 'cswTabs-middleborder'>
				##cswTabs-#elementID# > div > nav > ul > li.tab-current > a {
					border-left: #props.defaultBorder.size# solid #activeStruct.borderColor#;
					border-right: #props.defaultBorder.size# solid #activeStruct.borderColor#;
				}
				</cfif>
				
				##cswTabs-#elementID# > div > nav > ul > li:first-child > a {
					margin-left: 0;
				}
				##cswTabs-#elementID# > div > nav > ul > li:not(:first-child) > a {
					margin-left: -#props.defaultBorder.size#;
				}
			</cfoutput>

		<!--- BOTTOM LINE --->				
		<cfelseif props.tabType contains 'cswTabs-bottomline'>
			<cfoutput>
				##cswTabs-#elementID# > div > nav > ul > li > a {
					border-bottom: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#;
				}					
				
				/* hover */
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:hover,
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:focus {
					border-bottom: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#;
				}
			</cfoutput>
			
		<!--- SPACED BOTTOM LINE --->				
		<cfelseif props.tabType eq 'cswTabs-spacedbottomline'>
			<cfoutput>
				##cswTabs-#elementID# > div > nav > ul > li > a {
					border-bottom: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#;
				}					
				##cswTabs-#elementID# > div > nav > ul > li > a:hover::after,
				##cswTabs-#elementID# > div > nav > ul > li > a:focus::after,
				##cswTabs-#elementID# > div > nav > ul > li.tab-current a::after {
					background-color: #props.activeTab.bgColor#;
				}	
				
				/**
				##cswTabs-#elementID# > div > nav > ul > li > a::after {
					top: #props.defaultBorder.size#;
				}
				**/
				
				/* hover */
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:hover,
				##cswTabs-#elementID# > div > nav > ul > li:not(.tab-current) > a:focus {
					border-bottom: #props.defaultBorder.size# solid #props.activeTab.bgColor#;
				}
				
				##cswTabs-#elementID# > div > nav > ul > li.tab-current > a {
					border-bottom: #props.defaultBorder.size# solid #props.activeTab.bgColor#;
				}					
			</cfoutput>
			
		</cfif>
		
		
		<!--- bottom border extents full width --->
		<cfif props.tabBarLineFullWidth eq 1>
			<cfscript>
				lineColor = props.defaultBorder.borderColor;
				if( props.tabType eq 'cswTabs-folders' OR props.tabType eq 'cswTabs-slanted' )
					lineColor = props.defaultTab.bgColor;
			</cfscript>
			<cfoutput>
				/* full line */
				##cswTabs-#elementID# > div > nav > ul {
					border-bottom: #props.defaultBorder.size# solid #lineColor#;
				}	
				##cswTabs-#elementID# > div > nav > ul > li {
					margin-bottom: -#props.defaultBorder.size#;	
				}
			</cfoutput>
		</cfif>
		
		
		<!---// HIGHLIGHT STYLES //---------->		
		<cfif props.tabType contains 'cswTab-lineabove'>
			<cfoutput>
			##cswTabs-#elementID# > div > nav > ul > li.tab-current > a,
			##cswTabs-#elementID# > div > nav > ul > li.tab-current > a:hover,
			##cswTabs-#elementID# > div > nav > ul > li.tab-current > a:focus {
				box-shadow: inset 0px #props.highlightLine.size# 0px 0px #props.highlightLine.borderColor#;
				<cfif props.tabType contains 'cswTabs-fullborder'>
					border-top: #props.defaultBorder.size# solid #props.highlightLine.borderColor#;
					border-left: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#;
					border-right: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#;
					border-bottom: #props.defaultBorder.size# solid #props.activeTab.bgColor#;
				</cfif>
			}
			</cfoutput>
		</cfif>

		<cfif props.tabType contains 'cswTab-linebelow'>
			<cfoutput>
			##cswTabs-#elementID# > div > nav > ul > li.tab-current > a,
			##cswTabs-#elementID# > div > nav > ul > li.tab-current > a:hover,
			##cswTabs-#elementID# > div > nav > ul > li.tab-current > a:focus {
				box-shadow: inset 0px -#props.highlightLine.size# 0px 0px #props.highlightLine.borderColor#;
			}
			</cfoutput>
		</cfif>
		
		<cfif props.tabType contains 'cswTab-halfbelow'>
			<cfoutput>
			##cswTabs-#elementID# > div > nav > ul > li::before {		
			    position: absolute;
			    bottom: 0;
			    left: 50%;
			    width: 0%;
			    height: #props.highlightLine.size#;
			    background: #props.highlightLine.borderColor#;
			    content: '';
			    transition: all 0.3s ease-in;
				 z-index:100;
				 opacity: 0;
			}		
			##cswTabs-#elementID# > div > nav > ul > li.tab-current::before {		
			    left: 25%;
			    width: 50%;
	    	 	 transition: all 0.3s ease-in;
				 opacity: 1;
			}
			</cfoutput>		
		</cfif>
					               
		<cfif props.tabType contains 'cswTab-triangle'>
			<cfoutput>
			##cswTabs-#elementID# > div > nav > ul > li.tab-current::after {
			    border-top-color: #props.activeTab.bgColor#;
			}
			</cfoutput>		
		</cfif>


		<cfif props.tabType contains 'cswTab-circle'>
			<cfoutput>
				##cswTabs-#elementID# > div > nav > ul > li.tab-current > a {
					border: #props.highlightLine.size# solid #props.highlightLine.borderColor#;
				}	
			</cfoutput>
		</cfif>
		
		<cfif props.tabType EQ 'cswTabs-folders'>
			<cfoutput>
			##cswTabs-#elementID# > div > nav li > a {
				color: #props.defaultTab.color#;
			}			
			##cswTabs-#elementID# > div > nav li > a svg {
				fill: #props.defaultTab.bgColor#;   
			}		
			
			##cswTabs-#elementID# > div > nav li > a {
				background-color: #props.defaultTab.bgColor#;
			}
			##cswTabs-#elementID# > div > nav li > a:hover,
			##cswTabs-#elementID# > div > nav li > a:focus {
				background-color: #props.hoverTab.bgColor#;
			}
			##cswTabs-#elementID# > div > nav li > a:hover svg,
			##cswTabs-#elementID# > div > nav li > a:focus svg {
				fill: #props.hoverTab.bgColor#;
			}
			
			##cswTabs-#elementID# > div > nav li.tab-current > a {
				background: #props.activeTab.bgColor#;
			}
			##cswTabs-#elementID# > div > nav li.tab-current > a svg {
				fill: #props.activeTab.bgColor#;
			}
			</cfoutput>
		</cfif>
		
	</cffunction>
	
	
	<cffunction name="renderPaddingStyles" access="private" output="Yes" returntype="void">
		<cfargument name="propsStruct" type="struct" required="Yes">
	
		<cfscript>
			var padding = arguments.propsStruct.defaultPadding;
			var elementID = request.element.id;
			var top = 0;
			var right = 0;
			var bottom = 0;
			var left = 0;
			var padding_xxs = '';
			var padding_xs = '';
			var padding_sm = '';
			var padding_md = '';
			var p = StructNew();
			var units = 'px,%,em,rem,vw,vh,vmin,vmax';	
			var unit = 'px';
			var i = 0;
			
			if( padding eq '' )
				return;

			p.top = val( ListGetAt(padding, 1, ' ') );
			p.right = val( ListGetAt(padding, 2, ' ') );
			p.bottom = val( ListGetAt(padding, 3, ' ') );
			p.left = val( ListGetAt(padding, 4, ' ') );
				
			for( i=1; i lte ListLen(units); i=i+1 )
			{
				unit = ListGetAt(units, i);
				if( findNoCase(unit, padding) )
					break;
			}
			
			padding_xxs = GetScaledPadding(p, .4, unit);
			padding_xs = GetScaledPadding(p, .55, unit);
			padding_sm = GetScaledPadding(p, .7, unit);
			padding_md = GetScaledPadding(p, .85, unit);
		</cfscript>
		
		<cfoutput>
			<!--- XX-Small Devices --->
			@media (max-width: 479px) { 			
				##cswTabs-#elementID# > div > nav ul > li .cswInner { padding: #padding_xxs#; }
			}
			
			<!--- X-Small Devices --->
			@media (min-width: 480px) and (max-width: 767px)  { 
				##cswTabs-#elementID# > div > nav ul > li .cswInner { padding: #padding_xs#; }
			}

			<!--- Small devices --->
			@media (min-width: 768px) and (max-width: 991px)  { 			
				##cswTabs-#elementID# > div > nav ul > li .cswInner { padding: #padding_sm#; }
			}
			
			<!--- Medium devices --->
			@media (min-width: 992px) and (max-width: 1199px) { 			
				##cswTabs-#elementID# > div > nav ul > li .cswInner { padding: #padding_md#; }
			}

			<!--- Large devices --->
			@media (min-width: 1200px) { 		
				##cswTabs-#elementID# > div > nav ul > li .cswInner { padding: #padding#; }
			}
		</cfoutput>
		
	</cffunction>
	
	
	<cffunction name="GetScaledPadding" access="private" output="No" returntype="string">
		<cfargument name="padStruct" type="struct" required="yes">
		<cfargument name="scale" type="numeric" required="yes">
		<cfargument name="unit" type="string" required="yes">
		
		<cfscript>
			var u = arguments.unit;
			var retString = '';
			var p = arguments.padStruct;
			var newPad = StructNew();
			
			if( unit eq 'px' )
			{
				newPad.top = int(p.top * arguments.scale);
				newPad.right = int(p.right * arguments.scale);
				newPad.bottom = int(p.bottom * arguments.scale);
				newPad.left = int(p.left * arguments.scale);
			}
			else
			{
				newPad.top = NumberFormat(p.top * arguments.scale, '999.99');
				newPad.top = replaceNoCase( newPad.top, '.0', '' );
	
				newPad.right = NumberFormat(p.right * arguments.scale, '999.99');
				newPad.right = replaceNoCase( newPad.right, '.0', '' );
	
				newPad.bottom = NumberFormat(p.bottom * arguments.scale, '999.99');
				newPad.bottom = replaceNoCase( newPad.bottom, '.0', '' );
	
				newPad.left = NumberFormat(p.left * arguments.scale, '999.99');
				newPad.left = replaceNoCase( newPad.left, '.0', '' );			
			}
			
			retString = newPad.top & u & ' ' & newPad.right & u & ' ' & newPad.bottom & u & ' ' & newPad.left & u;
			
			return retString;
		</cfscript>
		
	</cffunction>
	
	
	<cffunction name="getBorderStyles" access="private" output="No" returntype="struct">
		<cfargument name="default" type="struct" required="yes">
		<cfargument name="cur" type="struct" required="yes">
		
		<cfscript>
			var retStruct = StructNew();
			 
			retStruct.size = arguments.default.size;
			
			if( arguments.cur.borderColor eq 'inherit' )
				retStruct.borderColor = arguments.default.borderColor;
			else
				retStruct.borderColor = arguments.cur.borderColor;
			
			return retStruct;			
		</cfscript>
		
	</cffunction>
	
</cfcomponent>